#include <OneWire.h>

#define pin_EC A1  // pin sur lequel la sonde est branchee 
#define VREF 5000.0 // definition de VREF pour la conversion en volts 
#define SCOUNT 30   // somme des points de mesure
         
int analogBuffer[SCOUNT]; // range les mesures dans un tableau   
int analogBufferTemp[SCOUNT];
int analogBufferIndex=0,copyIndex=0;
float averageVoltage=0,ECValue=0;  // assigne la temperature, a modifier si utilisation d'un thermometre 

int pin_T=2;  // pin sur lequel la sonde est branchee 

OneWire ds(pin_T);  // initialisation du capteur

void setup(){
  Serial.begin(9600); // frequence de communication avec le port serie
}

void loop(){

  float temperature = getTemp(); // mesure T
   
  static unsigned long analogSampleTimepoint = millis();
  if(millis()-analogSampleTimepoint > 40U) // toutes les 40 ms  
  {
    analogSampleTimepoint = millis();
    analogBuffer[analogBufferIndex] = analogRead(pin_EC);
    analogBufferIndex++;
    if(analogBufferIndex == SCOUNT) 
      analogBufferIndex = 0;
  }   
  static unsigned long printTimepoint = millis();
  if(millis()-printTimepoint > 800U)
  {
    printTimepoint = millis();
    for(copyIndex=0;copyIndex<SCOUNT;copyIndex++)
      analogBufferTemp[copyIndex]= analogBuffer[copyIndex];
    averageVoltage = getMedianNum(analogBufferTemp,SCOUNT)*(float)VREF/1024.0;  // signal en mV
    float compensationCoefficient=1.0+0.0185*(temperature-25.0);  // coeff compensation T
    float compensationVoltage=averageVoltage/compensationCoefficient; // applique la compensation en T
      
    Serial.print("Vcomp = ");
    Serial.print(compensationVoltage,2);  // affichage tension 
    Serial.print(" mV;  ");
    Serial.print("T = ");
    Serial.print(temperature); // affiche T
    Serial.println(" °C");
    
    delay(10);  // repete la mesure toutes les 10 ms
  }
}

int getMedianNum(int bArray[], int iFilterLen){
  int bTab[iFilterLen];
  for (byte i = 0; i<iFilterLen; i++)
    bTab[i] = bArray[i];
  int i, j, bTemp;
  for (j = 0; j < iFilterLen - 1; j++) 
  {
    for (i = 0; i < iFilterLen - j - 1; i++) 
    {
      if (bTab[i] > bTab[i + 1]) 
      {
        bTemp = bTab[i];
        bTab[i] = bTab[i + 1];
        bTab[i + 1] = bTemp;
      }
    }
  }
  if ((iFilterLen & 1) > 0)
    bTemp = bTab[(iFilterLen - 1) / 2];
  else
    bTemp = (bTab[iFilterLen / 2] + bTab[iFilterLen / 2 - 1]) / 2;
      
  return bTemp;
}

float getTemp(){  
  byte data[12];
  byte addr[8];

  if ( !ds.search(addr)) {
    ds.reset_search();
    return -1000;
  }

  if ( OneWire::crc8( addr, 7) != addr[7]) {
    Serial.println("CRC is not valid!");
    return -1000;
  }

  if ( addr[0] != 0x10 && addr[0] != 0x28) {
    Serial.print("Device is not recognized");
    return -1000;
  }

  ds.reset();
  ds.select(addr);
  ds.write(0x44,1); 

  byte present = ds.reset();
  ds.select(addr);
  ds.write(0xBE); 

  for (int i = 0; i < 9; i++) { 
    data[i] = ds.read();
  }

  ds.reset_search();

  byte MSB = data[1];
  byte LSB = data[0];

  float tempRead = ((MSB << 8) | LSB); 
  float TemperatureSum = tempRead / 16;

  return TemperatureSum;
}
